#include <CapacitiveSensor.h>
#include <Servo.h>


/*** Globals and constants ***/
// States
#define REST 0
#define ENTICING 1
#define BITE 2
int state = REST;

// Capacitive sensor
#define TOUCH_THRESH 300  // ADC value threshold to register a touch
#define TOUCH_NUM_FRAMES 30 // number of frames averaged before outputting the touch value
#define TOUCH_SEND_PIN 12
#define TOUCH_RECV_PIN 2
CapacitiveSensor chicken = CapacitiveSensor(TOUCH_SEND_PIN, TOUCH_RECV_PIN);
unsigned long chickenTouchVal = 0;

// Servos
#define PETAL_PIN 9
#define PETAL_DELAY 30 // milliseconds delay between each 1 degree change in servo position
#define PETAL_MIN 0
#define PETAL_MAX 90
#define JAW_PIN 5
#define JAW_BITE_DELAY 30 // milliseconds delay between each 1 degree change in servo position
#define JAW_RELAX_DELAY 50
#define JAW_MIN 0
#define JAW_MAX 90
Servo petalServo;
int posPetal = 0;
bool petalReverse = false;
Servo jawServo;
int posJaw = 0;

// PIR sensor
#define PIR_PIN 3
#define PIR_STABILIZE_TIME 12000 // milliseconds
#define ENTICE_TIMEOUT 5000 // milliseconds to wait, after person has left, before going back to REST state
unsigned long lastTrig = 0; // exact millisecond time at which PIR was last triggered
bool pirTrig = false;


/*** Setup code (executes once at the beginning) ***/
void setup() {
  // initialize serial port communication
  Serial.begin(9600);

  // disable capacitive touch sensor's autocalibration feature
  chicken.set_CS_AutocaL_Millis(0xFFFFFFFF);

  // attach the servo objects to the their pins
  petalServo.attach(PETAL_PIN);
  jawServo.attach(JAW_PIN);

  // give the PIR sensor time to stabilize
  delay(PIR_STABILIZE_TIME);
  Serial.println("PIR sensor stabilized");

  // set up an interrupt based on the PIR output's rising edge
  // the callback function is "pirDetected"
  attachInterrupt(digitalPinToInterrupt(PIR_PIN), pirDetected, RISING);
}


/*** Main loop code (loops forever) ***/
void loop() {
  /*** state machine ***/
  // REST state
  if (state == REST) {
    if (pirTrig == true) {
      state = ENTICING;
      pirTrig = false;
    }
  }

  // ENTICING state
  else if (state == ENTICING) {
    chickenTouchVal = chicken.capacitiveSensor(TOUCH_NUM_FRAMES);
    if (chickenTouchVal > TOUCH_THRESH)
      state = BITE;
    else if (pirTrig == false) {
      if ((millis() - lastTrig) > ENTICE_TIMEOUT)
        state = REST;
    }
    else if (pirTrig == true) {
      pirTrig = false;
    }

    petalServo.write(posPetal);
    if (posPetal == PETAL_MAX)
      petalReverse = true;
    else if (posPetal == PETAL_MIN)
      petalReverse = false;

    if (petalReverse == false)
      posPetal++;
    else
      posPetal--;
    delay(PETAL_DELAY);
  }

  // BITE state
  else if (state == BITE) {
    // Bite!
    for(posJaw = JAW_MAX; posJaw > JAW_MIN; posJaw--) {
      jawServo.write(posJaw);
      delay(JAW_BITE_DELAY);
      if(pirTrig == true)
        pirTrig = false;
    }
    // Relax jaw
    for(posJaw = JAW_MIN; posJaw < JAW_MAX+1; posJaw++) {
      jawServo.write(posJaw);
      delay(JAW_RELAX_DELAY);
      if(pirTrig == true)
        pirTrig = false;
    }
  }
}


/*** Callbacks and helper functions ***/
void pirDetected() {
  if (pirTrig == false) {
    pirTrig = true;
    lastTrig = millis();
  }
}






